==> buckshot-0.4.5.jar: buckshot/demo.clj <==
(ns buckshot.demo
  (:require [buckshot.backend :as backend]
            [buckshot.core :as buckshot]
            [buckshot.queue :as queue]
            [buckshot.tasks :as tasks]
            [buckshot.util :as util]
            [buckshot.worker :as worker]))

;; init
(def backend (backend/make-in-memory {}))

(def queue (queue/make {:backend backend}))

(buckshot/add-log-handler! queue println)

(def worker (worker/make {:id "worker 1"
                          :fns {:add (fn [w j] (apply + (:args j)))
                                :bad (fn [w j] (throw (Exception. "bad")))
                                :schedule-jobs tasks/schedule-jobs!}
                          :queue queue}))

(defn status-check []
  (println "specs:" (queue/specs queue))
  (println "scheduled:" (queue/scheduled queue))
  (println "processing:" (queue/processing queue)))

(status-check)

;; one-time task
(queue/subscribe! queue "good" #(buckshot/log! queue "result:" %))

(queue/add-specs! queue [{:id "good"
                          :fn :add
                          :args [1 1]}])

;; task exception
(queue/add-specs! queue [{:id "bad"
                          :fn :bad}])

(->> (queue/processing queue)
     first
     (queue/finish-job! queue))

;; recurring task
(queue/add-specs! queue [{:id "schedule jobs"
                          :fn :schedule-jobs
                          :period :second}])

(worker/stop! worker)

(->> (queue/specs queue)
     first
     (queue/remove-spec! queue))

(worker/start! worker)

;; unknown job should not block everything
(queue/add-specs! queue [{:id "ugly"
                          :fn :unknown}
                         {:id "good"
                          :fn :add
                          :args [2 2]}])

(->> (queue/specs queue)
     first
     (queue/remove-spec! queue))

(status-check)

==> buckshot-0.5.1.jar: buckshot/demo.clj <==
(ns buckshot.demo
  (:require [buckshot.backend :as backend]
            [buckshot.core :as buckshot]
            [buckshot.queue :as queue]
            [buckshot.tasks :as tasks]
            [buckshot.util :as util]
            [buckshot.worker :as worker]))

;; init
(def backend (backend/make-in-memory {}))

(def queue (queue/make {:backend backend}))

(buckshot/add-log-handler! queue println)

(def worker (worker/make {:id "worker 1"
                          :fns {:add (fn [w j] (apply + (:args j)))
                                :bad (fn [w j] (throw (Exception. "bad")))
                                :schedule-jobs tasks/schedule-jobs!}
                          :queue queue}))

(defn status-check []
  (println "specs:" (queue/specs queue))
  (println "scheduled:" (queue/scheduled queue))
  (println "processing:" (queue/processing queue)))

(status-check)

;; one-time task
(queue/subscribe! queue "good" #(buckshot/log! queue "result:" %))

(queue/add-spec! queue {:id "good"
                        :fn :add
                        :args [1 1]})

;; task exception
(queue/add-spec! queue {:id "bad"
                        :fn :bad})

(->> (queue/processing queue)
     first
     (queue/finish-job! queue))

;; recurring task
(queue/add-spec! queue {:id "schedule jobs"
                        :fn :schedule-jobs
                        :period :second})

(worker/stop! worker)

(->> (queue/specs queue)
     first
     (queue/remove-spec! queue))

(worker/start! worker)

;; unknown job should not block everything
(queue/add-spec! queue {:id "ugly"
                        :fn :unknown})

(queue/add-spec! queue {:id "good"
                        :fn :add
                        :args [2 2]})

(->> (queue/specs queue)
     first
     (queue/remove-spec! queue))

(status-check)
